import PyCubing
from PyCubing import GetScramble


def Test_Scrambles():
    # Test 2x2x2 Scramble
    print("\033[1;4m2x2x2's Scramble:\033[m \033[3m", end='')
    gen2x2x2: list = GetScramble.Cube2x2x2()
    for item in gen2x2x2:
        print(item, end=' ')
    print('\033[m')

    # Test 3x3x3 Scramble
    print("\033[1;4m3x3x3's Scramble:\033[m \033[3m", end='')
    gen3x3x3: list = GetScramble.Cube3x3x3()
    for item in gen3x3x3:
        print(item, end=' ')
    print('\033[m')

    # Test 4x4x4 Scramble
    print("\033[1;4m4x4x4's Scramble:\033[m \033[3m", end='')
    gen4x4x4: list = GetScramble.Cube4x4x4()
    for item in gen4x4x4:
        print(item, end=' ')
    print('\033[m')

    # Test 5x5x5 Scramble
    print("\033[1;4m5x5x5's Scramble:\033[m \033[3m", end='')
    gen5x5x5: list = GetScramble.Cube5x5x5()
    for item in gen5x5x5:
        print(item, end=' ')
    print('\033[m')

    # Test 6x6x6 Scramble
    print("\033[1;4m6x6x6's Scramble:\033[m \033[3m", end='')
    gen6x6x6: list = GetScramble.Cube6x6x6()
    for item in gen6x6x6:
        print(item, end=' ')
    print('\033[m')

    # Test 7x7x7 Scramble
    print("\033[1;4m7x7x7's Scramble:\033[m \033[3m", end='')
    gen7x7x7: list = GetScramble.Cube7x7x7()
    for item in gen7x7x7:
        print(item, end=' ')
    print('\033[m')

    # Test Pyraminx Scramble
    print("\033[1;4mPyraminx's Scramble:\033[m \033[3m", end='')
    genPyra: list = GetScramble.Pyraminx()
    for item in genPyra:
        print(item, end=' ')
    print('\033[m')

    # Test Skewb Scramble
    print("\033[1;4mSkewb's Scramble:\033[m \033[3m", end='')
    genSkewb: list = GetScramble.Skewb()
    for item in genSkewb:
        print(item, end=' ')
    print('\033[m')

    # Test Square One Scramble
    print("\033[1;4mSquare One's Scramble:\033[m \033[3m", end='')
    genSquareOne: list = GetScramble.Square_One()
    for item in genSquareOne:
        print(item, end=' ')
    print('\033[m')

    print('-' * 25)


def test_API() -> None:
    print('Top 10 WCA 3x3x3 average records')

    ranking: dict = PyCubing.get_wca_ranking()
    for rank in ranking:
        print(f'\033[1m{rank}:\033[m {ranking[rank]}')

    print('-' * 25)


if __name__ == '__main__':
    print(f'Pycubing Version: \033[1m{PyCubing.__version__}\033[m')
    Test_Scrambles()
    test_API()



import random
from PyCubing import GetScramble


def test_Cube3x3x3() -> None:
    """
    Test GetScramble.Cube3x3x3() function.
    """

    actual_output1: list = GetScramble.Cube3x3x3()
    actual_output2: list = GetScramble.Cube3x3x3()

    assert actual_output1 != actual_output2



##-*--------------- Scrambles ---------------*-##
#                                               #
#   Library create with the prupose to generate #
# scrambles for the WCA Puzzles. Fell free to   #
# copy or modify something here.                #
#                                               #
##-*-----------------------------------------*-##

from random import randint, choice

__all__: list = [
    'Cube2x2x2',
    'Cube3x3x3',
    'Cube4x4x4',
    'Cube5x5x5',
    'Cube6x6x6',
    'Cube7x7x7',
    'Pyraminx',
    'Skewb',
    'Square_One',
    'Megaminx',
    'Clock',
]


def sameAxis(MoveA: str, MoveB: str, MoveC: str) -> bool:
    """
    Internal function to check if a group of 3 movements
    are from the same axis.
    A sequence of 3 moves in the same aixis is not valid.
    Ex.: R L R, F B F, U D U, L R L, etc.
    """
    concatened: str = MoveA[-1] + MoveB[-1] + MoveC[-1]
    return concatened == 'xxx' or concatened == 'yyy' or concatened == 'zzz'


def Cube2x2x2(size: int = 10) -> list:
    """
    This function generates a scramble for 2x2x2 Cube
    Following the WCA guidelines.
    """

    Moves_types: tuple = ('R', 'U', 'F')
    Orientation: tuple = ('', "'", '2')

    MoveA: str = ''
    Moves: list = []

    for move in range(1, size + randint(0, 3)):
        while True:
            MoveB: str = Moves_types[randint(0, len(Moves_types) - 1)]
            if MoveB != MoveA:
                break

        MoveA: str = MoveB
        Moves.append(MoveB + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Cube3x3x3(size: int = 21) -> list:
    """
    This function generates a scramble for 3x3x3 Cube
    Following the WCA guidelines.
    """

    Moves_Types: tuple = ('Rx', 'Uy', 'Bz', 'Lx', 'Dy', 'Fz')
    Orientation: tuple = ('', "'", '2')

    MoveA: str = ' '
    MoveB: str = ' '
    Moves: list = []

    for move in range(1, size + randint(0, 5)):
        while True:
            MoveC: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if not sameAxis(MoveA, MoveB, MoveC) and MoveC != MoveB:
                break

        MoveA: str = MoveB
        MoveB: str = MoveC
        Moves.append(MoveC[0] + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Cube4x4x4(size: int = 40) -> list:
    """
    This function generates a scramble for 4x4x4 Cube
    Following the WCA guidelines.
    """

    Moves_Types: tuple = ('Rx', 'Uy', 'Bz', 'Lx', 'Dy', 'Fz')
    Orientation: tuple = ('', "'", '2', 'w', "w'", 'w2')

    MoveA: str = ' '
    MoveB: str = ' '
    Moves: list = []

    for move in range(1, size + randint(0, 5)):
        while True:
            MoveC: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if not sameAxis(MoveA, MoveB, MoveC) and MoveC != MoveB:
                break

        MoveA: str = MoveB
        MoveB: str = MoveC
        Moves.append(MoveC[0] + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Cube5x5x5(size: int = 55) -> list:
    """
    This function generates a scramble for 5x5x5 Cube
    Following the WCA guidelines.
    """

    Moves_Types: tuple = ('Rx', 'Uy', 'Bz', 'Lx', 'Dy', 'Fz')
    Orientation: tuple = ('', "'", '2', 'w', "w'", 'w2')

    MoveA: str = ' '
    MoveB: str = ' '
    Moves: list = []

    for move in range(0, size + randint(0, 5)):
        while True:
            MoveC: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if not sameAxis(MoveA, MoveB, MoveC) and MoveC != MoveB:
                break

        MoveA: str = MoveB
        MoveB: str = MoveC
        Moves.append(MoveC[0] + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Cube6x6x6(size: int = 55) -> list:
    """
    This function generates a scramble for 6x6x6 Cube
    Following the WCA guidelines.
    """

    Moves_Types: tuple = (
        'Rx',
        '3Rx',
        'Uy',
        '3Uy',
        'Bz',
        '3Bz',
        'Lx',
        '3Lx',
        'Dy',
        '3Dy',
        'Fz',
        '3Fz',
    )
    Orientation: tuple = ('', "'", '2', 'w', "w'", 'w2')

    MoveA: str = ' '
    MoveB: str = ' '
    Moves: list = []

    for move in range(0, size + randint(0, 7)):
        while True:
            MoveC: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if not sameAxis(MoveA, MoveB, MoveC) and MoveC != MoveB:
                break

        MoveA: str = MoveB
        MoveB: str = MoveC

        if MoveC[0] == '3':
            Moves.append(
                MoveC[0]
                + MoveC[1]
                + Orientation[randint(3, len(Orientation) - 1)]
            )
        else:
            Moves.append(
                MoveC[0] + Orientation[randint(0, len(Orientation) - 1)]
            )

    return Moves


def Cube7x7x7(size: int = 65) -> list:
    """
    This function generates a scramble for 7x7x7 Cube
    Following the WCA guidelines.
    """

    Moves_Types: tuple = (
        'Rx',
        '3Rx',
        'Uy',
        '3Uy',
        'Bz',
        '3Bz',
        'Lx',
        '3Lx',
        'Dy',
        '3Dy',
        'Fz',
        '3Fz',
    )
    Orientation: tuple = ('', "'", '2', 'w', "w'", 'w2')

    MoveA: str = ' '
    MoveB: str = ' '
    Moves: list = []

    for move in range(0, size + randint(0, 8)):
        while True:
            MoveC: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if not sameAxis(MoveA, MoveB, MoveC) and MoveC != MoveB:
                break

        MoveA: str = MoveB
        MoveB: str = MoveC

        if MoveC[0] == '3':
            Moves.append(
                MoveC[0]
                + MoveC[1]
                + Orientation[randint(3, len(Orientation) - 1)]
            )
        else:
            Moves.append(
                MoveC[0] + Orientation[randint(0, len(Orientation) - 1)]
            )

    return Moves


def Pyraminx(edges_size: int = 9, corner_sizes: int = 3):
    """
    This function generates a scramble for Pyraminx
    Following the WCA guidelines.
    """

    Moves_Types: tuple = ('R', 'L', 'U', 'B')
    Orientation: tuple = ('', "'")

    Moves: list = []

    # Edges moves
    MoveA: str = ''
    for move in range(1, edges_size + randint(0, 1)):
        while True:
            MoveB: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if MoveB != MoveA:
                break

        MoveA: str = MoveB
        Moves.append(MoveB + Orientation[randint(0, len(Orientation) - 1)])

    MoveA: str = ''
    for move in range(1, corner_sizes + randint(0, 1)):
        while True:
            MoveB: str = Moves_Types[randint(0, len(Moves_Types) - 1)].lower()
            if MoveB != MoveA:
                break

        MoveA: str = MoveB
        Moves.append(MoveB + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Skewb(size: int = 9) -> list:
    """
    This function generates a scramble for Skewb
    Following the WCA guidelines.
    """

    Moves_Types: tuple = ('R', 'L', 'U', 'B')
    Orientation: tuple = ('', "'")

    MoveA: str = ''
    Moves: list = []
    for move in range(1, size + randint(0, 1)):
        while True:
            MoveB: str = Moves_Types[randint(0, len(Moves_Types) - 1)]
            if MoveB != MoveA:
                break

        MoveA: str = MoveB
        Moves.append(MoveB + Orientation[randint(0, len(Orientation) - 1)])

    return Moves


def Square_One(size: int = 14) -> list:
    """
    This function generates a scramble for Square One
    Following the WCA guidelines.
    """

    MoveA: tuple = ()
    Moves: list = []

    for move in range(1, size + 1):
        while True:
            x: int = randint(-5, 6)
            y: int = randint(-5, 6)
            MoveB: tuple = (x, y)

            if MoveB != MoveA:
                break

        Moves.append((x, y))
        if not move == size:
            Moves.append('/')

    add_slash: bool = choice((True, False))
    if add_slash:
        Moves.append('/')

    return Moves


def Megaminx(size: int = 30) -> list:
    # TODO: Finish this part of code
    # TODO: Study how clock scramble works
    """
    This function generates a scramble for Magaminx
    Following the WCA guidelines.
    """
    return ['not working yet']


def Clock(size: int = 10) -> list:
    # TODO: Finish this part of code
    # TODO: Study how clock scramble works
    """
    This function generates a scramble for Clock
    Following the WCA guidelines.
    """
    return ['not working yet']


if __name__ == '__main__':
    print(
        'This is a module file, then you should import it.\n'
        + 'Plase, take a look in https://github.com/Samuel-de-Oliveira/PyRubik for more info'
    )



__all__: list = ['reverse', 'is_wca_notation']


def reverse(scramble: list) -> list:
    """
    Function that rewrite a scramble backwards
    Ex.:
        ["R", "U", "R'", "U", "D'"] -> ["D", "U'", "R", "U'", "R'"]
    """
    backwards_scramble: list = []
    for move in scramble:
        if move[-1] == "'":
            backwards_scramble.insert(0, move.replace("'", ''))
        else:
            backwards_scramble.insert(0, f"{move}'")

    return backwards_scramble


def is_wca_notation(scramble: list) -> bool:
    """
    Function that verify if a list is a official WCA notation.
    Usefull if someone digit a wrong notation on a input box, for
    example.
    """
    # Tuples with the valid notation for moves
    valid_notation_cubes: tuple = ('U', 'D', 'R', 'L', 'F', 'B')
    valid_notation_clock: tuple = ()
    valid_notation_megaminx: tuple = ()

    # TODO: Finish this part of code in next version

    return True


if __name__ == '__main__':
    print(
        'This is a module file, then you should import it.\n'
        + 'Plase, take a look in https://github.com/Samuel-de-Oliveira/PyRubik for more info'
    )



# Init file

# Constants
__version__: str = '0.1.0.2'
__author__: str = 'Samuel de Oliveira'
__license__: str = 'MIT'
__all__: list = []
__doc__: str = """
-*--------- The PyRubik official docs ---------*-

    Everything can be found at this link:
    https://github.com/Samuel-de-Oliveira/PyRubik

-*----------------------------------------------*-
"""

if __name__ == '__main__':
    print(
        'This is a module file, then you should import it.\n'
        + 'Plase, take a look in https://github.com/Samuel-de-Oliveira/PyRubik for more info'
    )



from setuptools import setup, find_packages
import PyRubik

# Get the README.md file and create the doc
with open('README.md', 'r', encoding='utf-8') as fh:
    long_description: str = fh.read()

# Get the project requirements
with open('requirements.txt', 'r', encoding='utf-8') as fh:
    project_requirements: str = fh.read()

# Setup
setup(
    name='PyRubik',
    version=PyRubik.__version__,
    author='Samuel de Oliveira',
    author_email='samwolfg12@gmail.com',
    packages=find_packages(),
    url='https://github.com/Samuel-de-Oliveira/PyCubing/',
    license='MIT',
    description='A Python module to make speedcubing projects a piece of cake.',
    keywords="cubing rubik rubik's cube solver scramble cube pyrubik",
    long_description=long_description,
    python_requires='>=3.10',
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Education',
        'Topic :: Games/Entertainment',
    ],
    entry_points={'console_scripts': []},  # comming soon...
)
